#
# This package contains the config, and it's persistence.
#
package GShowTV::Config;

use strict;
use warnings;
use Glib qw/TRUE FALSE/;
use GShowTV::Constants;
use GShowTV::Debug qw/debug/;

sub new {
	my $self = bless {};

	$self->{initial_size}   = undef;
	$self->{xmltvfile}      = undef;
	$self->{pvrinterface}   = undef;
	$self->{viewpastshows}  = FALSE;
	$self->{loadimages}     = FALSE;
	$self->{useimages}      = FALSE;
	$self->{hpanepos}       = undef;
	$self->{vpanepos}       = undef;
	$self->{dateformat}     = "%Y-%m-%d";
	$self->{use24htime}     = TRUE;
	$self->{remindertime}   = 5; 		# Deprecated since 1.1.0, remains for old configs sake
	$self->{usereminders}   = TRUE;		# Deprecated since 1.1.0, remains for old configs sake
	$self->{configversion}  = undef;
	$self->{view}           = undef;
	$self;
}

###############################################################################
# Configuration IO functions
###############################################################################

sub read_config {
	my $self = shift;
	
	if (open (READ, "<:utf8", $GShowTV::Constants::conffile)) {
		while (<READ>) {
			chomp();

			if (/^xmltvfile\s*=\s*(.*)/) {
				$self->{xmltvfile} = $1;
			} elsif (/^pvrinterface\s*=\s*(.*)/) {
				$self->{pvrinterface} = $1;
			} elsif (/^viewpastshows\s*=\s*(.*)/) {
				$self->{viewpastshows} = $1;
			} elsif (/^initial_size\s*=\s*(.*)/) {
				$self->{initial_size} = $1;
			} elsif (/^useimages\s*=\s*(.*)/) {
				$self->{useimages} = $1;
			} elsif (/^hpanepos\s*=\s*(.*)/) {
				$self->{hpanepos} = $1;
			} elsif (/^vpanepos\s*=\s*(.*)/) {
				$self->{vpanepos} = $1;
			} elsif (/^loadimages\s*=\s*(.*)/) {
				$self->{loadimages} = $1;
			} elsif (/^dateformat\s*=\s*(.*)/) {
				$self->{dateformat} = $1;
			} elsif (/^use24htime\s*=\s*(.*)/) {
				$self->{use24htime} = $1;
			} elsif (/^version\s*=\s*(.*)/) {
				$self->{configversion} = $1;
			} elsif (/^view\s*=\s*(.*)/) {
				$self->{view} = $1;
			}
		}
		close READ;
	}

	unless (defined ($self->{viewpastshows})) {
		$self->{viewpastshows} = FALSE;
	}
	unless (defined ($self->{useimages})) {
		$self->{useimages} = FALSE;
	}

	{ no warnings;
	  debug("Using configuration");
	  debug("XMLTV file:            $self->{xmltvfile}");
	  debug("PVR interface::        $self->{pvrinterface}");
	  debug("View past shows:       $self->{viewpastshows}");
	  debug("Use images:            $self->{useimages}");
	  debug("HPane position:        $self->{hpanepos}");
	  debug("VPane position:        $self->{vpanepos}");
	  debug("Load images:           $self->{loadimages}");
	  debug("Dateformat:            $self->{dateformat}");
	  debug("Use 24h time:          $self->{use24htime}");
	  debug("Use reminders:         $self->{usereminders}");
	  debug("reminder time:         $self->{remindertime}");
	  debug("Default view:          $self->{view}");
	  debug("Config version:        $self->{configversion}");
	  debug("");
	  debug("LANGUAGE    = " . $ENV{'LANGUAGE'});
	  debug("LANG        = " . $ENV{'LANG'});
	  debug("LC_MESSAGES = " . $ENV{'LC_MESSAGES'});
	  debug("");
	}
}

sub write_config {
	my $self = shift;
	
	if (open (WRITE, ">:utf8" , $GShowTV::Constants::conffile)) {
		print WRITE "version       = $GShowTV::Constants::VERSION\n";
		print WRITE "xmltvfile     = $self->{xmltvfile}\n"     if defined ($self->{xmltvfile});
		print WRITE "pvrinterface  = $self->{pvrinterface}\n"  if defined ($self->{pvrinterface});
		print WRITE "viewpastshows = $self->{viewpastshows}\n" if defined ($self->{viewpastshows});
		print WRITE "initial_size  = $self->{initial_size}\n"  if defined ($self->{initial_size});
		print WRITE "useimages     = $self->{useimages}\n"     if defined ($self->{useimages});
		print WRITE "hpanepos      = $self->{hpanepos}\n"      if defined ($self->{hpanepos});
		print WRITE "vpanepos      = $self->{vpanepos}\n"      if defined ($self->{vpanepos});
		print WRITE "loadimages    = $self->{loadimages}\n"    if defined ($self->{loadimages});
		print WRITE "dateformat    = $self->{dateformat}\n"    if defined ($self->{dateformat});
		print WRITE "use24htime    = $self->{use24htime}\n"    if defined ($self->{use24htime});
		print WRITE "view          = $self->{view}\n"          if defined ($self->{view});
		close WRITE;
	}
}

1;
