#
# This package contains constants used globally within the application. 
#
package GShowTV::Constants;

use strict;
use warnings;
use utf8;

use Locale::TextDomain qw/gshowtv/;

# Program name and version

our $name    = "GShowTV";
our $VERSION = "1.2.2";

## Configuration dir and file

our $confdir  = $ENV{'HOME'}."/.gshowtv";
our $imagedir = $confdir . "/images";
our $datadir  = $confdir . "/data";
our $conffile = $confdir . "/config";

our $favouritesfile = $confdir . "/favourites.dat";

# Installation prefix

our $PREFIX = "/usr";

# Images

our $default_channel_image = $PREFIX . "/share/gshowtv/undefined.png";
our $application_image     = $PREFIX . "/share/pixmaps/gshowtv.png";

# PVR Installation dir

our $pvr_installation_folder = $PREFIX . "/share/gshowtv/pvr";

# My weekdays

our @weekdays = (__"Sunday", __"Monday", __"Tuesday", __"Wednesday",
				 __"Thursday", __"Friday", __"Saturday", __"Sunday");

# My months

our @months = (__"January", __"February", __"March", __"April",
			   __"May", __"June", __"July", __"August",
			   __"September", __"October", __"November", __"December");

# My dateformats

our @dateformats = ("%Y-%m-%d",
					"%m/%d/%Y",
					"%m/%d",
					"%d.%m.%Y",
					"%d.%m");

# Notification sender

our $notify_send = `which notify-send`; chomp($notify_send); $notify_send = undef if ($notify_send eq "");

# XMLTV Druid

our $xmltv_druid = `which xmltv-druid`; chomp($xmltv_druid); $xmltv_druid = undef if ($xmltv_druid eq ""); 

# My run command

our $run_command;

# My about dialog data

our @authors  = ("Petri Vakevainen <Petri.Vakevainen\@iki.fi>");
our $comments = __"GShowTV is a program for viewing TV Show information and recording them. GShowTV can be configured to use any video recording backend and together with the backend GShowTV is a complete PVR solution.";
our $copyright = "Copyright © 2004,2005,2006,2007 Petri Väkeväinen";
our @artists   = ("Vyacheslav Dikonov <sdiconov\@mail.ru>");
our $website = "http://gshowtv.sourceforge.net/";

our $translator_credits = __"translator_credits"; $translator_credits = undef if (!defined ($translator_credits) || $translator_credits eq "translator_credits" || $translator_credits eq "");

# Localization status, the current language is translated or not?
our $LOCALIZATION_ENABLED = 0; $LOCALIZATION_ENABLED = 1 unless (__"TV Shows" eq "TV Shows");

1;
