package GShowTV::Data::IO;

use warnings;
use strict;

use Storable qw/fd_retrieve store_fd/;
use GShowTV::Constants;
use GShowTV::Debug qw/debug/;

sub read_favourites {
	my $config = shift;
	my $favourites;	
	
	my $oldfavouritesfile = $GShowTV::Constants::favouritesfile;
	$oldfavouritesfile =~ s/\.dat$//;
	
	if (!-e $GShowTV::Constants::favouritesfile && -e $oldfavouritesfile) {
		debug("Detected pre 1.1.0 style favourites file, reading it in");
		
		if (open (READ, "<:utf8", $oldfavouritesfile)) {
			while (<READ>) {
				chomp();
				my $f;

				if (/(.*):::(\d+),(\d+),(\d+)/) {
					$f->{name} = $1;
					$f->{color_r} = $2;
					$f->{color_g} = $3;
					$f->{color_b} = $4;
				} else {
					$f->{name} = $_;
					$f->{color_r} = undef;
					$f->{color_g} = undef;
					$f->{color_b} = undef;
				}
				$f->{mode}   = 1;
				$f->{inlist} = 1;
				$f->{reminder} = $config->{usereminders};
				$f->{time}     = $config->{remindertime};
				my ($r1, $r2);

				$r1->{value} = $f->{name};
				$r1->{field} = "title";
				$r1->{rule}  = "contains";

				$r2->{value} = $f->{name};
				$r2->{field} = "desc";
				$r2->{rule}  = "contains";

				$f->{rules}    = [$r1, $r2];
						
				push @$favourites, $f;

				debug("Favourite pattern: $_");
			}
			close READ;
			write_favourites();
		}
	} else {
		debug("read_favourites", "Detected 1.1.0 style favourites file, reading it in");
		if (open READ, "<:utf8", $GShowTV::Constants::favouritesfile) {
			$favourites = fd_retrieve *READ;
			close READ;
		}
	}
	$favourites;
}

sub write_favourites {
	my $favourites = shift;
	if (open (WRITE, ">:utf8", $GShowTV::Constants::favouritesfile)) {
		store_fd $favourites, *WRITE;
		close WRITE;
		my $oldfavouritesfile = $GShowTV::Constants::favouritesfile;
		$oldfavouritesfile =~ s/\.dat//;
		unlink $oldfavouritesfile;
	} else {
		debug("Cannot persist favourites.");
	}
}

sub write_channels {
	my $channels = shift;
	my $tree;

	# Make a clone
	
	foreach my $cid (keys %$channels) {
		my $orig = $channels->{$cid};
		my $c;
		foreach my $key (keys %$orig) {
			$c->{$key} = $orig->{$key} unless ($key eq "icon"); # Remove the icon reference
		}
		$tree->{$cid} = $c;
	}
	
	debug("Writing channels to file: $GShowTV::Constants::confdir/channels.dat");

	if (open (WRITE, ">:utf8", $GShowTV::Constants::confdir . "/channels.dat")) {
		store_fd $tree, *WRITE;
		close WRITE;
	} else {
		debug("Cannot persist channels.");
	}
}

sub read_channels {
	my $channel_tree;

	if (-e $GShowTV::Constants::confdir . "/channels.dat" && open (READ, "<:utf8", $GShowTV::Constants::confdir . "/channels.dat")) {
		$channel_tree = fd_retrieve *READ;
		close READ;
	} else {
		debug("Couldn't read channels.dat");
	}
	$channel_tree;
}

1;
