##############################################################################
# Handles reminders
# 

package GShowTV::Data::Reminderer;

use strict;
use warnings;

use Glib qw/TRUE FALSE/;
use GShowTV::Constants;
use GShowTV::Debug qw/debug/;
use GShowTV::Strings qw/str2time format_time/;

sub new {
	my $self = bless {};
	$self;
}

sub set_config {
	my $self        = shift;
	$self->{config} = shift;
	
	debug("Called");
}

sub set_channels {
	my $self = shift;
	$self->{channels} = shift;

}
sub set_data {
	my $self = shift;
	$self->{data} = shift;
}

sub start {
	my $self = shift;
	debug("Starting reminder notifier");

	my $sid = Glib::Timeout->add(1 * 60 * 1000, \&main_loop_iteration, $self);
	$self->{main_loop_id} = $sid;
	
	# Run the main loop once when starting.
	Glib::Timeout->add(1, sub {my $self = shift; $self->main_loop_iteration(); FALSE; }, $self);
}

sub main_loop_iteration {
	my $self = shift;

	return TRUE unless defined ($self->{data});
	
	for (my $i=0; $i < scalar @{$self->{data}}; $i++) {
		my $f = $self->{data}->[$i]->{f};
		my $p = $self->{data}->[$i]->{p};

		# TODO We don't really want to show reminders for shows that are in the past
		# Should it be handled here, or by the task that gives us the programs?
		
		if ((str2time("%Y%m%d%H%M%S", $p->{start}) - $f->{time} * 60) < time) {
			$self->notify($p, $f);
			splice @{$self->{data}}, $i, 1;
			$i--;
		}
	}

	TRUE;
}

sub notify {
	my $self = shift;
	my $p    = shift;
	my $f    = shift;

	debug("Showing notification for " . $p->{id});

	# Format the texts

	my $t = format_time($self->{config}, $p);
	my $d = "";
	$d = $p->{desc} if (defined ($p->{desc}));
	$d = $d . "\n\n" . $p->{category} if (defined ($p->{category}));

	my $title  = $p->{title};
	$title =~ s/&/&amp;/g;
	$title =~ s/</&lt;/g;
	$title =~ s/>/&gt;/g;

	my $c = $self->{channels}->{$p->{channel}}->{name};

	if (defined($GShowTV::Constants::notify_send)) {
		no warnings;
		# There's some overflow bug with the notify-send command, it cannot handle longer timeperiods than 35
		
		my @cmd = ("-t", ($f->{time} > 35 ? 35 : $f->{time}) * 60 * 1000, "-i", $GShowTV::Constants::application_image, $title, "$t $c&#10;&#10;$d");

		debug("exec: $GShowTV::Constants::notify_send " . $p->{id} . " " . ($f->{time} * 60 * 1000));

		my $child = fork();

		if ($child == 0) { # Child
			exec ($GShowTV::Constants::notify_send, @cmd);
			exit;
		} else { # Parent
			debug("Forked and continuing.");
		}
	} else {
		no warnings;
		my $dialog = Gtk2::MessageDialog->new_with_markup(undef,
			'destroy-with-parent',
			'info',
			'close',
			"<b><big>$title</big></b>");
		$dialog->format_secondary_markup("$t $c\n\n$d");
		$dialog->signal_connect(response => sub { $_[0]->hide; $_[0]->destroy; });
		$dialog->show();

		# Attach a timeout to the dialog to remove them after the notification time has run out
		Glib::Timeout->add($f->{time} * 60 * 1000, sub { if (defined ($_[0])) { $_[0]->hide(); $_[0]->destroy(); } FALSE; }, $dialog);
	}
}

1;
