package GShowTV::Favourite;

use warnings;
use strict;
use Glib qw/TRUE FALSE/;
use Gtk2;

use GShowTV::Debug qw/debug/;
# Container for all favourites

my @favourites = ();

sub new {
	my $self = bless {};
	push @favourites, $self;
	$self;
}

sub new_from_program {
	my $self = bless {};
	push @favourites, $self;
	my $p    = $_[1];
	
	my ($rule);
	
	$self->{name}   = $p->{title};
	$rule->{field} = "title";
	$rule->{rule}  = "contains";
	$rule->{value} = $p->{title};

	my @rules = ();
	push @rules, $rule;
	$self->{rules} = \@rules;	
	$self;
}

sub is_created_from_program {
	my $self = shift;
	my $p    = shift;

	# We assume the creation to be title contains, and if matches, then it's created from a program.
	return FALSE unless ($self->isFavourite($p));
	return FALSE unless (scalar @{$self->{rules}} == 1);

	my $rule = ${$self->{rules}}[0];
	
	return FALSE unless ($rule->{field} eq "title");
	return FALSE unless ($rule->{rule} eq "contains");
	return TRUE;
}

sub get_color {
	my $self = shift;
	if (defined ($self->{color_r}) && defined ($self->{color_g}) && defined ($self->{color_b})) {
		Gtk2::Gdk::Color->new($self->{color_r}, $self->{color_g}, $self->{color_b});
	} else {
		undef;
	}
}

sub isFavourite {
	my $self = shift;
	my $p    = shift;
	my @set  = undef;

	# Deal with both the possible caller syntaxes: 
	# $favourite->isFavourite($p)	- Object interface, test the "self" favourite
	# isFavourite($p)               - Static interface, test all the favourites

	unless (defined ($p)) {
		$p = $self;
		$self = undef;
	}

	if (defined ($self)) {
		push @set, $self;
	} else {
		@set = @favourites;
	} 

	foreach my $self (@set) {
		my $result = undef;
	
		foreach my $rule (@{$self->{rules}}) {
#			print "Testing " . $p->{title} . "against " . Dumper($rule) . "\n";
			my $cond  = $rule->{rule};    
			my $value = $rule->{value};
			my $re = "";

			if ($cond eq "is" || $cond eq "is not") {
				$re = "^\Q$value\E\$";
			} elsif ($cond eq "starts with" || $cond eq "does not start with") {
				$re = "^\Q$value\E";
			} elsif ($cond eq "ends with" || $cond eq "does not end with") {
				$re = "$\Qvalue\E\$";
			} elsif ($cond eq "contains" || $cond eq "does not contain") {
				$re = "$value";
			} else {
				print STDERR "Unknown condition $cond\n";
			}

			my $not = $cond =~ /not/;

			if (defined ($p->{$rule->{field}}) && $not && !($p->{$rule->{field}} =~ /$re/i)) {
				$result = $self;
			} elsif (defined ($p->{$rule->{field}}) && !$not && $p->{$rule->{field}} =~ /$re/i) {
				$result = $self;
			} else {
				$result = undef;
			}

			if ($self->{mode} == 0 && !$result) { # ALL CRITERIA MUST BE TRUE
				return undef;
			} elsif ($self->{mode} == 1 && $result) { # ANY CRITERIA IS ENOUGH
				return $self;
			}
		}
		return $self if ($result);
	}
	return undef;
}

# Reads in the plain object array and converts the objects to the favourite objects
sub deserialize {
	my $favourites = GShowTV::Data::IO::read_favourites();
	my @wrapped;
	
	foreach my $f (@$favourites) {
		my $w = GShowTV::Favourite->new();
		
		foreach my $c (keys %$f) {
			$w->{$c} = $f->{$c};
		}
		push @wrapped, $w;
	}
	\@wrapped;
}

# Converts the favourites to a plain object array and writes them to a file
sub serialize {
	my $favs = shift;
	my @unwrapped;
	
	foreach my $f (@$favs) {
		my $w;

		foreach my $c (keys %$f) {
			$w->{$c} = $f->{$c};
		}
		push @unwrapped, $w;
	}
	GShowTV::Data::IO::write_favourites(\@unwrapped);
}

1;
