#
# This is the main application which handles the stuff to deal with Gtk2
#
package GShowTV::UI::Application;

use utf8;
use Glib qw(TRUE FALSE);
use Gtk2;
use Gtk2::SimpleList;
use Gtk2::Gdk::Keysyms;

use strict;
use warnings;

use GShowTV::UI::Favourite::ListDialog;
use GShowTV::UI::ChannelDialog;
use GShowTV::UI::GladeWrapper;
use GShowTV::UI::ConfigDialog;
use GShowTV::UI::ListView;
use GShowTV::UI::GridView;
use GShowTV::Favourite;
use GShowTV::Constants;
use GShowTV::Debug qw/debug/;
use GShowTV::Strings qw/str2time time2str format_right_notebooktime myglob/;
use Locale::TextDomain qw/gshowtv/;

use Data::Dumper;


use Glib::Object::Subclass
    Glib::Object::, # Parent class
	signals => {
		quit => {
			method      => 'default_signal_handler',
			flags       => [qw/run-first/],
			return_type => undef, # void return
			param_types => [qw/Glib::Scalar/], # instance and data are automatic
		},
		record => {
			method      => 'default_signal_handler',
			flags       => [qw/run-first/],
			return_type => undef, # void return
			param_types => [qw/Glib::Scalar/], # instance and data are automatic
		},
		cancel => {
			method      => 'default_signal_handler',
			flags       => [qw/run-first/],
			return_type => undef, # void return
			param_types => [qw/Glib::Scalar/], # instance and data are automatic
		},
		watch => {
			method      => 'default_signal_handler',
			flags       => [qw/run-first/],
			return_type => undef, # void return
			param_types => [qw/Glib::Scalar/], # instance and data are automatic
		},
		nautilus => {
			method      => 'default_signal_handler',
			flags       => [qw/run-first/],
			return_type => undef, # void return
			param_types => [qw/Glib::Scalar/], # instance and data are automatic
		},
		delete => {
			method      => 'default_signal_handler',
			flags       => [qw/run-first/],
			return_type => undef, # void return
			param_types => [qw/Glib::Scalar/], # instance and data are automatic
		},
		find => {
			method      => 'default_signal_handler',
			flags       => [qw/run-first/],
			return_type => undef, # void return
			param_types => [qw/Glib::Scalar/], # instance and data are automatic
		},
		xmltv_druid_activated => {
			method      => 'default_signal_handler',
			flags       => [qw/run-first/],
			return_type => undef, # void return
			param_types => [qw//], # instance and data are automatic
		},
		channels_changed => {
			method      => 'default_signal_handler',
			flags       => [qw/run-first/],
			return_type => undef, # void return
			param_types => [qw/Glib::Scalar/], # instance and data are automatic
		},
		config_changed => {
			method      => 'default_signal_handler',
			flags       => [qw/run-first/],
			return_type => undef, # void return
			param_types => [qw/Glib::Scalar/], # instance and data are automatic
		},
		favourites_changed => {
			method      => 'default_signal_handler',
			flags       => [qw/run-first/],
			return_type => undef, # void return
			param_types => [qw/Glib::Scalar/], # instance and data are automatic
		},
		program_selected => {
			method      => 'do_program_selected',
			flags       => [qw/run-first/],
			return_type => undef, # void return
			param_types => [qw/Glib::Scalar Glib::Scalar/], # instance and data are automatic
		},
		channel_selected => {
			method      => 'default_signal_handler',
			flags       => [qw/run-first/],
			return_type => undef, # void return
			param_types => [qw/Glib::Scalar/], # instance and data are automatic
		},
	};

sub INIT_INSTANCE {
    my $self = shift;

	my $glade = GShowTV::UI::GladeWrapper->get_glade();
	$glade->signal_autoconnect_from_package($self);

    # Set the glade window
	
    $self->{window} = $glade->get_widget("application");
    $self->{window}->set_icon(Gtk2::Gdk::Pixbuf->new_from_file($GShowTV::Constants::application_image));
       
	# Get components
	
  	$self->{descriptionlabel} = $glade->get_widget('label');
  	$self->{statusbar}        = $glade->get_widget('statusbar');

	$self->{findentry}          = $glade->get_widget('findentry');

	# Grid/List mode selection elements 
    $self->{listviewmenuitem}   = $glade->get_widget('list_view');
	$self->{gridviewmenuitem}   = $glade->get_widget('grid_view');
	$self->{radiolistbutton}    = $glade->get_widget('listradiotoolbutton');
	$self->{radiogridbutton}    = $glade->get_widget('gridradiotoolbutton');

	# Right Treeviews

	$self->constructRightNotebookList($glade, "recordingtreeview");
	$self->constructRightNotebookList($glade, "recordedtreeview");
	$self->constructRightNotebookList($glade, "favouritestreeview");
	$self->constructRightNotebookList($glade, "findtreeview");
	
	$self->{recordingtreeview}->signal_connect("cursor-changed"  => \&on_cursor_changed, $self );
	$self->{recordedtreeview}->signal_connect("cursor-changed"   => \&on_cursor_changed, $self );
	$self->{favouritestreeview}->signal_connect("cursor-changed" => \&on_cursor_changed, $self );
	$self->{findtreeview}->signal_connect("cursor-changed"       => \&on_cursor_changed, $self );

	# Panes

	$self->{hpaned} = $glade->get_widget("hpaned1");
	$self->{vpaned} = $glade->get_widget("vpaned1");

	$self->{toolbar}      = $glade->get_widget("toolbar");

	# Program views

	$self->{listnotebook} = $glade->get_widget("listnotebook");
	$self->{listnotebook}->remove_page(0); # Remove the placeholder element

	$self->{listview}     = GShowTV::UI::ListView->new();
	$self->{gridview}     = GShowTV::UI::GridView->new();
	$self->{listnotebook}->append_page($self->{listview});
	$self->{listnotebook}->append_page($self->{gridview});
	
	$self->{listview}->signal_connect("program_selected" => sub { my ($l, $p, $self) = @_; $self->signal_emit("program_selected", $p, $l); }, $self);
	$self->{listview}->signal_connect("channel_selected" => sub { my ($l, $c, $self) = @_; $self->signal_emit("channel_selected", $c); }, $self);
	$self->{gridview}->signal_connect("program_selected" => sub { my ($g, $p, $self) = @_; $self->signal_emit("program_selected", $p, $g); }, $self);
	$self->{gridview}->signal_connect("channel_selected" => sub { my ($g, $c, $self) = @_; $self->signal_emit("channel_selected", $c); }, $self);
	$self->{listview}->signal_connect("button-press-event" => \&popupmenu, $self);
	$self->{gridview}->signal_connect("button-press-event" => \&popupmenu, $self);
	
	# Enable druid if the druid is available

	$glade->get_widget('xmltv_druid')->set_sensitive($GShowTV::Constants::xmltv_druid ne "");

	$self->{window}->signal_connect (destroy => sub { $_[1]->on_quit(); }, $self);
	
	# Enable size setting to config

	$self->{window}->signal_connect (configure_event => sub {
		my ($win, $event, $self) = @_;
		$self->{config}->{initial_size} = $event->width."x".$event->height;
		$self->{config}->{hpanepos}     = $self->{hpaned}->get_position();
		$self->{config}->{vpanepos}     = $self->{vpaned}->get_position();
		FALSE;
	}, $self);
	
	
}

sub show_all {
	my $self = shift;
	$self->{window}->show_all();
}

sub hide {
	my $self = shift;
	$self->{window}->hide();
}

sub constructRightNotebookList {
	my ($self, $glade, $id) = @_;

	$self->{$id} = Gtk2::SimpleList->new_from_treeview($glade->get_widget($id),
							    __"Date"    => 'text',
							    __"Channel" => 'text',
							    __"Program" => 'text',
							    'program'   => 'hidden',
							    'color'     => 'color');
	
    $self->{$id}->set_rules_hint(TRUE);
    $self->{$id}->set_headers_clickable(TRUE);

	foreach my $col ($self->{$id}->get_columns) {
		foreach my $cell ($col->get_cell_renderers) {
			$col->add_attribute ($cell, cell_background_gdk => 4);
		}
	}

	my $i = 0;
	while (defined ($self->{$id}->get_column($i))) {
		my $column = $self->{$id}->get_column($i);
		$column->set_sort_column_id($i++);
	}

	$self->{$id}->get_model()->set_sort_func(0, sub {
		my $list = shift;
		my $a = $list->get(shift, 3);
		my $b = $list->get(shift, 3);
                                        
		return 1 unless defined ($a);
		return -1 unless defined ($b);

		$a =~ m/(\d+)$/;
		$a = $1;
		$b =~ m/(\d+)$/;
		$b = $1;

		$a <=> $b;
	});

	$self->{$id}->set_search_column(0);
	
	$self->{$id}->signal_connect("button_press_event" => \&popupmenu, $self);
}

sub on_fullscreen_activate {
	my $self = shift;

	# Check if we think we are the accel group, then we need to rethink ourselves
	if ($self->isa("Gtk2::AccelGroup")) {
		$self = $self->{application};
	} elsif ($self->isa("Gtk2::MenuItem")) {
		$self = $self->parent()->{application};
	}
	
	# For fullscreen to be meaningful, we take the listnotebook off the application and put it
	# to a separate window that is fullscreened. 
	
	if (defined ($self->{fullscreen}) && $self->{fullscreen}->visible()) {
		debug("Going to un-fullscreen $self");

		$self->{fullscreen}->remove($self->{listnotebook});
		$self->{listnotebookparent}->add($self->{listnotebook});
		$self->{fullscreen}->hide();
		$self->show_all();
	} else {
		debug("going to fullscreen $self");

		unless (defined ($self->{fullscreen})) {
			$self->{fullscreen} = Gtk2::Window->new();
			$self->{fullscreen}->set_default_icon_from_file($GShowTV::Constants::application_image);
			$self->{fullscreen}->fullscreen();
			$self->{listnotebookparent} = $self->{listnotebook}->parent();

			my $accel = Gtk2::AccelGroup->new();
			$accel->{application} = $self;
			$accel->connect($Gtk2::Gdk::Keysyms{Escape},      [], 'visible', \&on_fullscreen_activate);
			$accel->connect($Gtk2::Gdk::Keysyms{F5},          [], 'visible', \&on_fullscreen_activate);

			$self->{fullscreen}->add_accel_group($accel);

			# TODO Fullscreen needs now and prev accelerators still
			debug("REQUIRES NOW AND PREV ACCELERATORS STILL");

			#$accel->connect($Gtk2::Gdk::Keysyms{KP_Add},      [], 'visible', \&on_nextdate_clicked);
			#$accel->connect($Gtk2::Gdk::Keysyms{KP_Substract},[], 'visible', \&on_prevdate_clicked);
			#$accel->connect($Gtk2::Gdk::Keysyms{KP_Enter},    [], 'visible', \&on_now_clicked);
		}
		$self->{listnotebookparent}->remove($self->{listnotebook});
		$self->{fullscreen}->add($self->{listnotebook});
		$self->hide();
		$self->{fullscreen}->show_all();
	} 
}

sub on_toolbar_visible_activate {
	my $self = shift;

	if ($self->{toolbar}->visible()) {
		$self->{toolbar}->hide;
	} else {
		$self->{toolbar}->show;
	}
}

sub on_grid_view_activate {
	my $self = shift;
	if (!defined ($self->{viewlock})) {
    	$self->{viewlock} = 1;
		$self->{listviewmenuitem}->set_active(FALSE);
		$self->{radiolistbutton}->set_active(FALSE);
		$self->{gridviewmenuitem}->set_active(TRUE);
		$self->{radiogridbutton}->set_active(TRUE);
		$self->{listnotebook}->show_all;
		$self->{listnotebook}->set_current_page(1);
		$self->{viewlock} = undef;
		$self->{config}->{view} = "grid";
	}
}

sub on_list_view_activate {
	my $self = shift;
	if (!defined ($self->{viewlock})) {
		$self->{viewlock} = 1;
		$self->{listviewmenuitem}->set_active(TRUE);
		$self->{radiolistbutton}->set_active(TRUE);
		$self->{gridviewmenuitem}->set_active(FALSE);
		$self->{radiogridbutton}->set_active(FALSE);
		$self->{listnotebook}->set_current_page(0);
		$self->{viewlock} = undef;
		$self->{config}->{view} = "list";
	}
}

sub on_key_press_event {
	my ($self, $widget, $event) = @_;

	# The magic number is enter or return, however you prefer it.

	if ("key-press" eq $event->type() && 65293 == $event->keyval()) {
		debug("enter pressed");
		if ($widget == $self->{findentry}) {
			$self->on_findbutton_clicked();
		} else {
			debug("unknown widget called us.");
		}
	}
}

sub on_prevdate_clicked {
	my ($self, $button) = @_;

	my @dates = sort keys %{$self->{program_tree}};

	for (my $i=0; $i < scalar @dates; $i++) {
		if ($dates[$i] eq $self->{current_date} && $i-1 >= 0) {
			$self->set_current_date($dates[$i-1]);
			last;
		}
	}
}

sub on_nextdate_clicked {
	my ($self, $button) = @_;

	my @dates = sort keys %{$self->{program_tree}};

	for (my $i=0; $i < scalar @dates; $i++) {
		if ($dates[$i] eq $self->{current_date} && $i+1 < scalar @dates) {
			$self->set_current_date($dates[$i+1]);
			last;
		}
	}
}

sub on_now_clicked {
	my ($self, $button) = @_;
	
	$self->set_current_date(time2str("%Y%m%d", time));
}

sub on_record {
	my ($self, $button) = @_;

	# Check if we think we are the accel group, then we need to rethink ourselves
	if ($self->isa("Gtk2::MenuItem")) {
		$self = $self->parent()->{application};
	}
	
	unless (defined ($self->{current_program})) {
		$self->set_statusbar_text(__"No program selected!");
		return;
	}

	$self->signal_emit("record", $self->{current_program});
}

sub on_cancel {
	my ($self, $button) = @_;

	# Check if we think we are the accel group, then we need to rethink ourselves
	if ($self->isa("Gtk2::MenuItem")) {
		$self = $self->parent()->{application};
	}
	
	my $index = ($self->{recordingtreeview}->get_selected_indices())[0];

	unless (defined ($index)) {
		$self->set_statusbar_text(__"No program selected from the recording list");
		return;
	}

	$self->signal_emit("cancel", $self->{current_program});
}

sub on_nautilus {
	my ($self, $button) = @_;

	$self->signal_emit("nautilus", $self->{current_program});
}

sub on_watch {
	my ($self, $button) = @_;

	# Check if we think we are the accel group, then we need to rethink ourselves
	if ($self->isa("Gtk2::MenuItem")) {
		$self = $self->parent()->{application};
	}

	my $index = ($self->{recordedtreeview}->get_selected_indices())[0];

	unless (defined ($index)) {
		$self->set_statusbar_text(__"No program selected from the recorded list");
		return;
	}

	$self->signal_emit("watch", $self->{current_program});
}

sub on_delete {
	my ($self, $button) = @_;
	
	# Check if we think we are the accel group, then we need to rethink ourselves
	if ($self->isa("Gtk2::MenuItem")) {
		$self = $self->parent()->{application};
	}
	
	my $index = ($self->{recordedtreeview}->get_selected_indices())[0];

	unless (defined ($index)) {
		$self->set_statusbar_text(__"No program selected from the recorded list");
		return;
	}

	$self->signal_emit("delete", $self->{current_program});
}

sub on_quit {
	my $self = shift;
	
	$self->hide();
	$self->signal_emit("quit", $self->{config});
}

sub on_preferences_activate {
	my $self = shift;

	my $dialog = GShowTV::UI::ConfigDialog->new();
	$dialog->set_config($self->{config});
	$dialog->show_all();
	my $res = $dialog->run();
	$dialog->hide();
	
	if ($res eq "ok") {
		my $config    = $dialog->get_config();
		# Copy over non config dialog entries
		$config->{initial_size} = $self->{config}->{initial_size};
		$config->{hpanepos} = $self->{config}->{hpanepos};
		$config->{vpanepos} = $self->{config}->{vpanepos};
		$config->{view}     = $self->{config}->{view};
		$self->set_config($config);
		$self->signal_emit("config_changed", $config);
	}
	$dialog->destroy();	
}

sub on_favourites_activate {
	my $self = shift;
	
	debug("Launching dialog");
	
	my $dialog = GShowTV::UI::Favourite::ListDialog->new(title => __"TV Shows - Favourites", channels => $self->{channels});
	$dialog->set_data($self->{favourites});
	
	$dialog->show_all();

	$dialog->signal_connect(response => sub {
		my $dialog = shift;
		$dialog->hide();

		if ($dialog->is_dirty()) {
			my $f = $dialog->get_data();
			$self->set_favourites($f);
			$self->signal_emit("favourites_changed", $f);
		}
		$dialog->destroy();
	});
	debug("Done");
}

sub on_information {
	my $self = shift;

	# TODO Include scrollkeeper support (OMF) -> Gnome2::Help
	
	my $language = $ENV{"REQUESTED_LANGUAGE"}; # We have set this environment value, a copy of LANGUAGE
	my $locale   = "C";

	debug("Find documentation for languages in $language");

	foreach my $loc (split /:/, $language) {
		$loc =~ s/\..*//;
		if ( -e "/usr/share/gnome/help/gshowtv/" . $loc . "/gshowtv.xml") {
			$locale = $loc;
			last;
		}
		$loc =~ s/_.*//;
		if ( -e "/usr/share/gnome/help/gshowtv/" . $loc . "/gshowtv.xml") {
			$locale = $loc;
			last;
		}
	}
	debug("Showing help for $locale locale");
	system("gnome-help /usr/share/gnome/help/gshowtv/" . $locale . "/gshowtv.xml &");
}

sub on_about {
	my $self = shift;
	my $dialog = Gtk2::AboutDialog->new();
	$dialog->signal_connect (response => sub { $_[0]->destroy });
  
	$dialog->set_authors(@GShowTV::Constants::authors);
	$dialog->set_artists(@GShowTV::Constants::artists);
	$dialog->set_comments($GShowTV::Constants::comments);
	$dialog->set_copyright($GShowTV::Constants::copyright);
	$dialog->set_logo(Gtk2::Gdk::Pixbuf->new_from_file($GShowTV::Constants::application_image));
	$dialog->set_translator_credits($GShowTV::Constants::translator_credits);
	$dialog->set_version($GShowTV::Constants::VERSION);
	$dialog->set_website($GShowTV::Constants::website);
	$dialog->set_name($GShowTV::Constants::name);
	$dialog->show_all();
}

sub on_channel_configuration_activate {
	my $self = shift;
	
	debug("Launching dialog");
	
	my $dialog = GShowTV::UI::ChannelDialog->new();
	$dialog->set_channels($self->{channels});
	
	$dialog->show_all();

	$dialog->signal_connect(response => sub {
		my $dialog = shift;
		my $res    = shift;

		$dialog->hide();
		
		if ($res eq "ok") {
			my $c = $dialog->get_channels();
			$self->set_channels($c);
			$self->signal_emit("channels_changed", $c);
		}

		$dialog->destroy();
	});
	debug("Done");
}

sub on_xmltv_druid_activate {
	my $self = shift;
	$self->signal_emit("xmltv_druid_activated");
}

sub on_findbutton_clicked {
	my $self = shift;

	my $string = $self->{findentry}->get_text();

	if (length($string) < 1) {
		debug("no search string");
		return;
	}

	$self->signal_emit("find", $string);
}

sub on_cursor_changed {
	my ($list, $self) = @_;

	debug("Called with $self and $list");

	# The components used by me are cleared via signal handling (program_selected), 
	# but the right notebook lists I have to manage here
	$self->unselect_all_but($list);

 	my $index = ($list->get_selected_indices())[0];

	my $pid = $list->{'data'}[$index][3];
	debug("Selected index $index -> $pid");

	$pid =~ /(.*)\.(\d{8})\d+/;

	my $p;

	if ($list == $self->{recordingtreeview} || $list == $self->{recordedtreeview}) {
		$p = $self->{pvr_tree}->{$pid};
	} else {
		$p = $self->{program_tree}->{$2}->{$1}->{$pid};
	}
	$self->signal_emit("program_selected", $p, $self);
}

sub on_add_to_favourites {
	my ($self, $button) = @_;
	
	# Check if we think we are the accel group, then we need to rethink ourselves
	if ($self->isa("Gtk2::MenuItem")) {
		$self = $self->parent()->{application};
	}

	debug("Launching dialog");
	
	my $dialog = GShowTV::UI::Favourite::FavouriteDialog->new(title => __"TV Shows - Favourites", channels => $self->{channels});

	$dialog->set_data(GShowTV::Favourite->new_from_program($self->{current_program}));
	$dialog->show_all();

	$dialog->signal_connect(response => sub {
		my $dialog   = shift;
		my $response = shift;
		
		$dialog->hide();

		if ($response eq "ok") {
			my $f = $dialog->get_data();
			push @{$self->{favourites}}, $f;
			$self->signal_emit("favourites_changed", $self->{favourites});
		}
		$dialog->destroy();
	});
	debug("Done");
}

sub on_remove_from_favourites {
	my ($self, $button) = @_;
	
	# Check if we think we are the accel group, then we need to rethink ourselves
	if ($self->isa("Gtk2::MenuItem")) {
		$self = $self->parent()->{application};
	}

	for (my $i = 0; $i < scalar @{$self->{favourites}}; $i++) {
		my $f = $self->{favourites}[$i];
		
		if ($f->is_created_from_program($self->{current_program})) {
			debug("Is created from program!");

			splice @{$self->{favourites}}, $i, 1;
			$self->signal_emit("favourites_changed", $self->{favourites}); # TODO Change this just to remove the favourites list, and recolor things
			last;
		}
	}

	debug("We are: " . $self);
}

sub unselect_all_but {
	my $self = shift;
	my $but  = shift;
	
	my @lists = ($self->{recordingtreeview}, $self->{recordedtreeview}, $self->{favouritestreeview}, $self->{findtreeview});

	# Clear the selections of right notebook lists

	foreach my $l (@lists) {
		$l->get_selection()->unselect_all() if (! defined ($but) || $l != $but);
	}
}


sub popupmenu {
	my ($source, $event, $self) = @_;
	
	if ($event->button != 3) {
		return;
	}

	debug("Called popupmenu with $source $event $self");

	my $menu = Gtk2::Menu->new();
	$menu->{application} = $self;

	my $isFavourite = FALSE;

	foreach my $f (@{$self->{favourites}}) {
		if ($f->isFavourite($self->{current_program})) {
			$isFavourite = TRUE;
			last;
		}
	}
	
	my $item = undef;

	if (!$isFavourite) {
		$item = Gtk2::ImageMenuItem->new(__"Add to favourites");
		$item->set_image(Gtk2::Image->new_from_stock('gtk-add', 'menu'));
		$item->signal_connect("activate" => \&on_add_to_favourites);
		$menu->append($item);
	}
	if ($isFavourite) {
		$item = Gtk2::ImageMenuItem->new(__"Remove from favourites");
		$item->set_image(Gtk2::Image->new_from_stock('gtk-remove', 'menu'));
		$item->signal_connect("activate" => \&on_remove_from_favourites);
		$menu->append($item);
	}
	if ($source != $self->{recordedtreeview} && $source != $self->{recordingtreeview}) {
		$item = Gtk2::ImageMenuItem->new(__"Record");
		$item->set_image(Gtk2::Image->new_from_stock('gtk-media-record', 'menu'));
		$item->signal_connect("activate" => \&on_record);
		$menu->append($item);
	}
	if ($source == $self->{recordingtreeview}) {
		$item = Gtk2::ImageMenuItem->new(__"Cancel");
		$item->set_image(Gtk2::Image->new_from_stock('gtk-cancel', 'menu'));
		$item->signal_connect("activate" => \&on_cancel);
		$menu->append($item);
	}
	if ($source == $self->{recordedtreeview}) {
		$item = Gtk2::ImageMenuItem->new(__"Delete");
		$item->set_image(Gtk2::Image->new_from_stock('gtk-delete', 'menu'));
		$item->signal_connect("activate" => \&on_delete);
		$menu->append($item);
		$item = Gtk2::ImageMenuItem->new(__"Watch");
		$item->set_image(Gtk2::Image->new_from_stock('gtk-zoom-fit', 'menu'));
		$item->signal_connect("activate" => \&on_watch);
		$menu->append($item);
	}
	if (defined ($self->{fullscreen}) && $self->{fullscreen}->visible()) {
	    $item = Gtk2::ImageMenuItem->new(__"Exit fullscreen");
	    $item->set_image(Gtk2::Image->new_from_stock('gtk-fullscreen', 'menu'));
	    $item->signal_connect("activate" => \&on_fullscreen_activate);
	    $menu->append($item);
	}
	$menu->popup(undef, undef, undef, undef, 3, 0);
	$menu->show_all();
}

sub default_signal_handler {
	# Dummy default implementation for event handler
}

sub set_config {
	my $self   = shift;
	my $config = shift;
	my $oldconfig = $self->{config};
	$self->{config} = $config;
		
	if (!defined ($oldconfig)) {
		if (defined ($config->{view}) && $config->{view} eq "grid") {
			$self->on_grid_view_activate();
		}
		if (defined ($config->{initial_size})) {
			$config->{initial_size} =~ m/(\d+)x(\d+)/;
			$self->{window}->resize($1,$2);
		}
		if (defined ($config->{hpanepos})) {
			$self->{hpaned}->set_position($config->{hpanepos});
		}
		if (defined ($config->{vpanepos})) {
			$self->{vpaned}->set_position($config->{vpanepos});
		}
	}

	$self->{listview}->set_config($config);
	$self->{gridview}->set_config($config);

	if (defined ($oldconfig)) {
		if ($config->{use24htime} != $oldconfig->{use24htime} || 
			$config->{dateformat} ne $oldconfig->{dateformat}) {
			$self->redraw_right_notebook_lists();
		}
	}
		
}

sub set_favourites {
	my $self = shift;
	$self->{favourites} = shift;
	
	$self->{listview}->set_favourites($self->{favourites});
	$self->{gridview}->set_favourites($self->{favourites});
}

sub set_statusbar_text {
	my $self = shift;
	my $msg  = shift;
	
	debug($msg) if defined $msg && $msg ne "";

	if (defined ($msg)) {
		$self->{statusbar}->push(0, $msg);
	} else {
		$self->{statusbar}->push(0, "");
	}
}

sub set_title {
	my $self = shift;
	my $title = shift;

	$self->{window}->set_title($title);
}

sub set_program_tree {
	my $self = shift;
	$self->{program_tree} = shift;
	$self->{listview}->set_program_tree($self->{program_tree});
	$self->{gridview}->set_program_tree($self->{program_tree});
	debug("Program tree is set.");
}

sub set_channels {
	my $self = shift;
	$self->{channels} = shift;
	my $noflush = shift;

	foreach my $c (values %{$self->{channels}}) {
		if (! defined ($c->{icon})) {
			my $filename = myglob ($GShowTV::Constants::imagedir . "/" . $c->{id} .".*");

			if (defined ($filename) && -e $filename) {
				$c->{icon} = $filename;
			} else {
				$c->{icon} = $GShowTV::Constants::default_channel_image;
			}
		}
	}

	$self->{listview}->set_channels($self->{channels}, $noflush);
	$self->{gridview}->set_channels($self->{channels}, $noflush);
	debug("Channels set.");
}

sub set_pvr_tree {
	my $self = shift;
	my $tree = shift;

	@{$self->{recordedtreeview}->{data}}  = ();
	@{$self->{recordingtreeview}->{data}} = ();

	# TODO This is not the perfect place for this, as this is replicated code form the programlistingtask
	foreach my $p (values %$tree) {
		my $list;
		
		if (defined ($p->{file}) && $p->{file} ne "") {
			$list = $self->{recordedtreeview};
		} else {
			$list = $self->{recordingtreeview};
		}
		my $color = undef;

		LOOP: foreach my $f (@{$self->{favourites}}) {
			if ($f->isFavourite($p)) {
				$color = $f->get_color();
				last LOOP;
			}
		}

		my $channelname = $self->{channels}->{$p->{channel}}->{name};
		
		push @{$list->{data}},
			[
			format_right_notebooktime($self->{config}, $p),
			$channelname,
			$p->{'title'},
			$p->{channel} . "." . $p->{start},
			$color
			];
	}

	$self->{pvr_tree} = $tree;
	
	debug("Pvr tree is set.");
}

sub set_current_date {
	my $self = shift;
	$self->{current_date} = shift;

	debug("current date is " . $self->{current_date});

	$self->{listview}->set_current_date($self->{current_date});
	$self->{gridview}->set_current_date($self->{current_date});

	my $t = str2time("%Y%m%d", $self->{current_date});

	$self->set_title(sprintf(__"TV Shows - %s %s %s", time2str("%A", $t) , time2str("%d", $t), time2str("%B", $t)));
		
}

sub set_description {
	my $self = shift;
	my $desc = shift;
	
	$self->{descriptionlabel}->set_text($desc);
}

sub do_program_selected {
	my $self   = shift;
	my $p      = shift;
	my $source = shift;
	
	if ($source == $self) {
		$self->{listview}->unselect_all();
		$self->{gridview}->unselect_all();
	} elsif ($source == $self->{listview}) {
		$self->{gridview}->unselect_all();
		$self->unselect_all_but(undef);
	} else {
		$self->unselect_all_but(undef);
		$self->{listview}->unselect_all();
	}
	
	$self->{current_program} = $p;
	my $desc = $p->{'desc'};

	if (defined ($p->{'category'})) {
		$desc = $desc . "\n\n" . $p->{'category'};
	}

	$self->set_description($desc);
	$self->set_statusbar_text("");
}

sub redraw_right_notebook_lists {
	my $self = shift;

	debug("Redrawing notebook lists, applying date display changes");

	foreach my $list ($self->{findtreeview}, $self->{favouritestreeview}, $self->{recordingtreeview}, $self->{recordedtreeview}) {
		foreach my $row (@{$list->{data}}) {
			my $pid = $$row[3];

			$pid =~ /(.*)\.(\d{8})\d\d\d\d\d\d/;

			my $p;
			
			if ($list == $self->{recordingtreeview} || $list == $self->{recordedtreeview}) {
				$p = $self->{pvr_tree}->{$pid};
			} else {
				$p = $self->{program_tree}->{$2}->{$1}->{$pid};
			}

			$$row[0] = format_right_notebooktime($self->{config}, $p),
		}
		$list->columns_autosize();
	}
}

1;

# TODO Make a signal handler listen to the favourites changed events and handle it our selves. The main handler shouldn't give it back to us.
