#
# This is the channel dialog
#
package GShowTV::UI::ChannelDialog;

use Glib qw(TRUE FALSE);
use Gtk2;
use Gtk2::SimpleList;

use strict;
use warnings;

use GShowTV::UI::GladeWrapper;
use GShowTV::Constants;
use GShowTV::Debug qw/debug/;

use Locale::TextDomain qw/gshowtv/;

use Glib::Object::Subclass
    Gtk2::Dialog::;

sub INIT_INSTANCE {
    my $self = shift;

	my $glade = GShowTV::UI::GladeWrapper->get_glade();

    # Remove glade container window

    my $g_container = $glade->get_widget("channeldialog");
    my $child = ($g_container->get_children())[0];
    $g_container->remove($child);

    # Set the glade object into me

    $self->vbox->add($child);
    
	$self->set_title(__"TV Shows - Channels");
	$self->set_default_size(500, 400);
	$self->set_icon(Gtk2::Gdk::Pixbuf->new_from_file($GShowTV::Constants::application_image));

	$self->{channeltreeview} = Gtk2::SimpleList->new_from_treeview($glade->get_widget("channelconftreeview"),
							     __"Visible"  => 'bool',
							     __"Name"     => 'text',
							     __"XMLTV ID" => 'text',
							     __"PVR ID"   => 'text',
							     __"Image"    => 'pixbuf',
							     "filename"   => 'hidden',
							     "id"         => 'hidden');
	$self->{channeltreeview}->set_column_editable(1, TRUE);
	$self->{channeltreeview}->set_column_editable(3, TRUE);

	$self->{imagedialog} = Gtk2::FileChooserDialog->new(__"Choose channel image",
							 undef,
							 'open',
							 'gtk-cancel' => 'cancel',
							 'gtk-ok' => 'ok');
	$self->{imagedialog}->signal_connect (response => sub {
			my ($dialog, $response, $info) = @_;
			if ($response eq "ok") {
				$dialog->{list}->{data}[$dialog->{row}][4] =
				Gtk2::Gdk::Pixbuf->new_from_file($dialog->get_filename());
				$dialog->{list}->{data}[$dialog->{row}][5] = $dialog->get_filename()
			}
			$dialog->hide;
		});

	$self->{channeltreeview}->signal_connect('button-press-event' => 
		sub {
			my ($self, $event, $dialog) = @_;

			return FALSE unless ("2button-press" eq $event->type());

			my ($path, $column, $cell_x, $cell_y) = 
				$self->get_path_at_pos ($event->x, $event->y);
			if ($path) {
				if (scalar ($self->get_selected_indices()) > 0 &&
					       ($self->get_selected_indices())[0] == $path->to_string() &&
							$column == $self->get_column(4)) {
					$dialog->{row}  = $path->to_string();
					$dialog->{list} = $self;
					my $cid  = $self->{data}[$path->to_string][2];
					my $icon = $self->{data}[$path->to_string][5];

					if ($icon eq $GShowTV::Constants::default_channel_image) {
						$dialog->set_current_folder($ENV{'HOME'});
					} else {
						$icon =~ s#/([^/]+)$##;
						$dialog->set_current_folder($icon);
					}
					$dialog->run();
				}
			}
			return FALSE;
		}, $self->{imagedialog});

    # Add my buttons

    $self->add_buttons('gtk-cancel' => 'cancel', 'gtk-ok' => 'ok');
}
				      
sub set_channels {
	my $self = shift;
	my $tree = shift;
	$self->{channels} = $tree;
		
	@{$self->{channeltreeview}->{data}} = ();

	foreach my $c (sort { $a->{order} <=> $b->{order} } values %$tree) {
		my $img = $GShowTV::Constants::default_channel_image;
		
		if (defined ($c->{icon}) && -e $c->{icon}) {
			$img = $c->{icon};
		}
		debug($c->{name} . " using image '" . $img);
		my $pixbuf = Gtk2::Gdk::Pixbuf->new_from_file($img);

		push @{$self->{channeltreeview}->{data}}, [$c->{visible}, $c->{name}, $c->{id}, $c->{pvrid}, $pixbuf, $img];
	}
}

sub get_channels {
	my $self = shift;

	my $channels;
	
	for (my $i = 0 ; $i < @{$self->{channeltreeview}->{'data'}}; $i++) {
	    my $c = { id      => $self->{channeltreeview}->{data}[$i][2],
				  pvrid   => $self->{channeltreeview}->{data}[$i][3],
				  name    => $self->{channeltreeview}->{data}[$i][1],
				  visible => $self->{channeltreeview}->{data}[$i][0],
				  order   => $i
            };

		# If the user set up the channel image, then save it to our image dir.

		if ($self->{channeltreeview}->{data}[$i][5] ne $GShowTV::Constants::default_channel_image &&
			!(($self->{channeltreeview}->{data}[$i][5]) =~ ($GShowTV::Constants::imagedir . "/" . $c->{id}))) {
			debug("Saving user selected channel image " . $self->{channeltreeview}->{data}[$i][5]);
			$self->{channeltreeview}->{data}[$i][4]->save($GShowTV::Constants::imagedir . "/" . $c->{id} . ".png", "png");
		}

		debug("Adding channel " . $c->{id} . " order " . $c->{order});
		$channels->{$c->{id}} = $c;
	}
	$channels;
}

1;
				      