#
# This is the config dialog
#
package GShowTV::UI::ConfigDialog;

use Glib qw(TRUE FALSE);
use Gtk2;

use strict;
use warnings;

use GShowTV::UI::GladeWrapper;
use GShowTV::Constants;
use GShowTV::Debug qw/debug/;
use GShowTV::Strings qw/time2str str2time/;

use Locale::TextDomain qw/gshowtv/;

use Glib::Object::Subclass
    Gtk2::Dialog::;

my $time = str2time("%Y%m%d", "20051231");

sub INIT_INSTANCE {
    my $self = shift;

	my $glade = GShowTV::UI::GladeWrapper->get_glade();

    # Remove glade container window

    my $g_container = $glade->get_widget("configdialog");
    my $child = ($g_container->get_children())[0];
    $g_container->remove($child);

    # Set the glade object into me

    $self->vbox->add($child);
    
	$self->set_title(__"TV Shows - Configuration");
	$self->set_default_size(400, 300);
	$self->set_icon(Gtk2::Gdk::Pixbuf->new_from_file($GShowTV::Constants::application_image));

    # Config components

	$self->{xmltvfileentry}     = $glade->get_widget('xmltvfileentry');
	$self->{pvrinterfaceentry}  = $glade->get_widget('pvrinterfaceentry');
	$self->{viewpastshowscheck} = $glade->get_widget('viewpastshowscheckbutton');
	$self->{useimagescheck}     = $glade->get_widget('useimagescheckbutton');
	$self->{loadimagescheck}    = $glade->get_widget('loadimagescheckbutton');
	$self->{use24htimecheck}    = $glade->get_widget('24hcheckbutton');
	$self->{dateformatcombobox} = $glade->get_widget('dateformatcombobox');

	foreach my $df (@GShowTV::Constants::dateformats) {
		$self->{dateformatcombobox}->append_text(time2str($df, $time));
	}
	
    # Add my buttons

    $self->add_buttons('gtk-cancel' => 'cancel', 'gtk-ok' => 'ok');
}

sub set_config {
	my $self   = shift;
	my $config = shift;
	$self->{config} = $config;
	
	$self->{xmltvfileentry}->set_filename($config->{xmltvfile}) if defined ($config->{xmltvfile});
	$self->{pvrinterfaceentry}->set_filename($config->{pvrinterface}) if defined ($config->{pvrinterface});
	$self->{viewpastshowscheck}->set_active($config->{viewpastshows});
	$self->{useimagescheck}->set_active($config->{useimages});
	$self->{loadimagescheck}->set_active($config->{loadimages});
	$self->{use24htimecheck}->set_active($config->{use24htime});
	my $combo =$self->{dateformatcombobox};
	
	my $i = 0;
	foreach my $df (@GShowTV::Constants::dateformats) {
		$combo->remove_text(0);
	}
	foreach my $df (@GShowTV::Constants::dateformats) {
		$combo->append_text(time2str($df, $time));
		
		if ($df eq $config->{dateformat}) {
			$self->{dateformatcombobox}->set_active($i);
		}
		$i++;
	}
}

sub get_config {
	my $self   = shift;
	my $c = GShowTV::Config->new();
	
	$c->{xmltvfile}     = $self->{xmltvfileentry}->get_filename();
	$c->{pvrinterface}  = $self->{pvrinterfaceentry}->get_filename();
	$c->{viewpastshows} = $self->{viewpastshowscheck}->get_active();
	$c->{useimages}     = $self->{useimagescheck}->get_active();
	$c->{loadimages}    = $self->{loadimagescheck}->get_active();
	$c->{use24htime}    = $self->{use24htimecheck}->get_active();

	foreach my $df (@GShowTV::Constants::dateformats) {
		my $str = time2str($df, $time);
		if ($str eq $self->{dateformatcombobox}->get_active_text()) {
			$c->{dateformat} = $GShowTV::Constants::dateformats[$self->{dateformatcombobox}->get_active()];
			last;
		}		
	}
	$c;
}

1;
