package GShowTV::UI::Favourite::FavouriteDialog;

use Glib qw/TRUE FALSE/;
use Gtk2;

use GShowTV::Debug qw/debug/;
use GShowTV::Favourite;
use GShowTV::UI::GladeWrapper;
use GShowTV::UI::Favourite::RuleRow;

use strict;
use warnings;

use Glib::Object::Subclass
    Gtk2::Dialog::, # Parent class
    properties => [
		   Glib::ParamSpec->scalar ('title',
					    'Title',
					    'The title',
					    [qw/readable writable/]),
		   Glib::ParamSpec->scalar ('channels',
					    'Channels',
					    'The channels tree',
					    [qw/readable writable/]),
	];

sub INIT_INSTANCE {
    my $self = shift;

	my $glade = GShowTV::UI::GladeWrapper->get_glade();

    # Remove glade container window

    my $g_container = $glade->get_widget("favouritesdialogwindow");
    my $child = ($g_container->get_children())[0];
    $g_container->remove($child);

    # Set the glade object into me

    $self->vbox->add($child);
    
    # Add my buttons

    $self->add_buttons('gtk-cancel' => 'cancel', 'gtk-ok' => 'ok');

    # Connect the add button
    my $addbutton = $glade->get_widget("addrulebutton");
    $addbutton->signal_connect(clicked => \&on_addbutton_clicked, $self);

    $self->{nameentry}                = $glade->get_widget("nameentry");
    $self->{criteriamodecombobox}     = $glade->get_widget("criteriamodecombobox");
    $self->{criteriavbox}             = $glade->get_widget("criteriavbox");
    $self->{includeinlistcheckbutton} = $glade->get_widget("includeinlistcheckbutton");
    $self->{showremindercheckbutton}  = $glade->get_widget("showremindercheckbutton");
    $self->{whentoremindspinbutton}   = $glade->get_widget("whentoremindspinbutton");
    $self->{colorbutton}              = $glade->get_widget("colorbutton");
	$self->{colorbutton}->set_color(Gtk2::Gdk::Color->new(255 * 257, 255 * 257, 255 * 257));

	# Make us notice that color is set

	$self->{colorbutton}->signal_connect("color-set" => sub {
		my ($button, $self) = @_;
		$self->{colorchanged} = TRUE;
		debug("color_set");
	}, $self);

    # Set things not settable via glade
    $self->{criteriamodecombobox}->set_active(0);
    $self->add_rulerow();

    $self->set_icon_from_file($GShowTV::Constants::application_image);
}

sub SET_PROPERTY {
    my ($self, $pspec, $newval) = @_;

    $self->{$pspec->get_name} = $newval;
    $self->set_title($newval) if ($pspec->get_name eq "title");
    
	foreach my $r ($self->{criteriavbox}->get_children()) {
		$r->SET_PROPERTY($pspec, $newval);
    }
}

sub on_addbutton_clicked {
    my ($button, $self) = @_;
    my $rule = $self->add_rulerow();
    $rule->show_all();
}

sub add_rulerow {
    my $self = shift;
    my $rule = GShowTV::UI::Favourite::RuleRow->new(channels => $self->{channels});
    
    $self->{criteriavbox}->pack_start($rule, FALSE, FALSE, 5);
    $rule;
}

sub get_data {
    my $self = shift;
    my $p = GShowTV::Favourite->new();

    $p->{name} = $self->{nameentry}->get_text();
    $p->{mode} = $self->{criteriamodecombobox}->get_active();

    $p->{inlist}   = $self->{includeinlistcheckbutton}->get_active();
    $p->{reminder} = $self->{showremindercheckbutton}->get_active();
    $p->{time}     = $self->{whentoremindspinbutton}->get_value();

	if ($self->{colorchanged}) {
	    $p->{color_r}  = $self->{colorbutton}->get_color()->red;
	    $p->{color_g}  = $self->{colorbutton}->get_color()->green;
	    $p->{color_b}  = $self->{colorbutton}->get_color()->blue;
	} else {
	    $p->{color_r}  = undef;
	    $p->{color_g}  = undef;
	    $p->{color_b}  = undef;
	
	}
    my @rules = ();

    foreach my $r ($self->{criteriavbox}->get_children()) {
	push @rules, $r->get_data();
    }

    $p->{rules} = \@rules;
    $p;
}

sub set_data {
	my $self = shift;
	my $p    = shift;
	
	$self->{nameentry}->set_text($p->{name}) if (defined ($p->{name}));
	$self->{criteriamodecombobox}->set_active($p->{mode}) if (defined $p->{mode});
	$self->{includeinlistcheckbutton}->set_active($p->{inlist}) if (defined $p->{inlist});
	$self->{showremindercheckbutton}->set_active($p->{reminder}) if (defined $p->{reminder});
	$self->{whentoremindspinbutton}->set_value($p->{time}) if (defined $p->{time});

	if (defined $p->{color_r} && defined $p->{color_g} && defined $p->{color_b}) {
		my $color = Gtk2::Gdk::Color->new($p->{color_r}, $p->{color_g}, $p->{color_b});
		$self->{colorbutton}->set_color($color);
		$self->{colorchanged} = TRUE;
	}

	if (defined $p->{rules}) {	
		$self->{criteriavbox}->foreach(sub {
			my ($child, $vbox) = @_;
			$vbox->remove($child);
		}, $self->{criteriavbox});

		foreach my $rule (@{$p->{rules}}) {
			my $row = $self->add_rulerow();
			$row->set_data($rule);
		}
	}
}

1;
