package GShowTV::UI::Favourite::ListDialog;
# Responsible for showing the favourites list dialog

use Glib qw(TRUE FALSE);
use Gtk2;
use Gtk2::SimpleList;

use Locale::TextDomain qw/gshowtv/;

use GShowTV::Debug qw/debug/;
use GShowTV::UI::GladeWrapper;
use GShowTV::UI::Favourite::FavouriteDialog;

use strict;
use warnings;

use Glib::Object::Subclass
    Gtk2::Dialog::, # Parent class
    properties => [
		   Glib::ParamSpec->scalar ('title',
					    'Title',
					    'The title',
					    [qw/readable writable/]),
		   Glib::ParamSpec->scalar ('channels',
					    'Channels',
					    'The channels tree',
					    [qw/readable writable/]),
		   Glib::ParamSpec->scalar ('dirty',
					    'Dirty',
					    'Dirty status of the favourites',
					    [qw/readable writable/]),
		   ];

sub INIT_INSTANCE {
    my $self = shift;

	my $glade = GShowTV::UI::GladeWrapper->get_glade();

    # Remove glade container window

    my $g_container = $glade->get_widget("favouriteslistdialogwindow");
    my $child = ($g_container->get_children())[0];
    $g_container->remove($child);

    # Set the glade object into me

    $self->vbox->add($child);
    
    # Add my buttons

    $self->add_buttons('gtk-close' => 'close');

    # Connect the buttons
    my $addbutton = $glade->get_widget("addfavbutton");
    $addbutton->signal_connect(clicked => \&on_addbutton_clicked, $self);
    my $editbutton = $glade->get_widget("editbutton");
    $editbutton->signal_connect(clicked => \&on_editbutton_clicked, $self);
    my $removebutton = $glade->get_widget("removebutton");
    $removebutton->signal_connect(clicked => \&on_removebutton_clicked, $self);

    $self->{treeview} = Gtk2::SimpleList->new_from_treeview($glade->get_widget("treeview"),
							    'Name'  => 'text',
							    'color' => 'color');
	foreach my $col ($self->{treeview}->get_columns) {
        foreach my $cell ($col->get_cell_renderers) {
                $col->add_attribute ($cell, cell_background_gdk => 1);
        }
	}
	$self->{dirty} = FALSE;
    $self->set_icon_from_file($GShowTV::Constants::application_image);
}

sub SET_PROPERTY {
    my ($self, $pspec, $newval) = @_;

    $self->{$pspec->get_name} = $newval;
    $self->set_title($newval) if ($pspec->get_name eq "title");
}

sub set_data {
    my $self = shift;
	my $data = shift;
	
    foreach my $fav (@$data) {
    	$self->add_data($fav);
    } 
    $self->{treeview}->unselect(0);
}

sub add_data {
	my $self = shift;
	my $fav  = shift;

	my $color = undef;

	if (defined ($fav->{color_r}) && defined ($fav->{color_g}) && defined ($fav->{color_b})) {
		$color = Gtk2::Gdk::Color->new($fav->{color_r}, $fav->{color_g}, $fav->{color_b});
	}	

	push @{$self->{treeview}->{data}}, [$fav->{name}, $color];
	push @{$self->{data}}, $fav;
}

sub get_data {
	my $self = shift;
	$self->{data};
}

sub is_dirty {
	my $self = shift;
	$self->{dirty};
}

sub on_addbutton_clicked {
    my ($button, $self) = @_;

    my $dialog = GShowTV::UI::Favourite::FavouriteDialog->new(title => __"TV Shows - Favourite", 
															  channels => $self->{channels});

	$dialog->show_all();
	my $response = $dialog->run();
	$dialog->hide();
	
	if ($response eq "ok") {
		$self->add_data($dialog->get_data());
		$self->{dirty} = TRUE;
	}
}

sub on_editbutton_clicked {
    my ($button, $self) = @_;
    my $index = ($self->{treeview}->get_selected_indices())[0];
    
    return unless defined ($index);

    my $dialog = GShowTV::UI::Favourite::FavouriteDialog->new(title      => __"TV Shows - Favourite", channels => $self->{channels});
	$dialog->set_data($self->{data}->[$index]);
	
	$dialog->show_all();
	my $response = $dialog->run();
	$dialog->hide();
	
	if ($response eq "ok") {
		my $fav = $dialog->get_data();
		my $color = undef;

		if (defined ($fav->{color_r}) && defined ($fav->{color_g}) && defined ($fav->{color_b})) {
			$color = Gtk2::Gdk::Color->new($fav->{color_r}, $fav->{color_g}, $fav->{color_b});
		}

		$self->{data}->[$index] = $fav;
		$self->{treeview}->{data}->[$index][0] = $fav->{name};
		$self->{treeview}->{data}->[$index][1] = $color;
		$self->{dirty} = TRUE;
	}
}

sub on_removebutton_clicked {
    my ($button, $self) = @_;
    
    my $index = ($self->{treeview}->get_selected_indices())[0];
    
    if (defined ($index)) {
    	$self->{treeview}->unselect($index);
		splice @{$self->{treeview}->{data}}, $index, 1;
		splice @{$self->{data}}, $index, 1;
    }
    
    $self->{dirty} = TRUE;
}

1;
