package GShowTV::UI::Favourite::RuleRow;

use Glib qw(TRUE FALSE);
use Gtk2;
use Data::Dumper;
use Locale::TextDomain qw/gshowtv/;

use strict;
use warnings;

use Glib::Object::Subclass
    Gtk2::HBox::, # Parent class
    properties => [
		   Glib::ParamSpec->scalar ('channels',
					    'Channels',
					    'The channels tree',
					    [qw/readable writable/]),
	];

# These are containers for the mappings of internal text to localized text.    
my @RR_ALLRULES    = ( {"contains"    => __"contains" },
					{"does not contain"    => __"does not contain" },
				    {"is"          => __"is" }, 
				    {"is not"      => __"is not" },
				    {"starts with" => __"starts with" },
				    {"ends with"   => __"ends with" }, 
				    {"does not start with" => __"does not start with" }, 
				    {"does not end with"   => __"does not end with" });
my @RR_SMALLRULES  = ( {"is" => __"is" }, {"is not" => __"is not" });
my @RR_FIELDS      = ( {"title" => __"Title" }, {"desc" => __"Description" }, {"channel" => __"Channel" });
#{"category" => __"Category" });

sub INIT_INSTANCE {
    my $self = shift;

    $self->{field} = Gtk2::ComboBox->new_text();
    foreach my $field (get_translated_text(\@RR_FIELDS)) {
	    $self->{field}->append_text($field);
    }
    $self->{field}->set_active(0);
    $self->{field}->signal_connect(changed => \&on_change, $self);
    $self->pack_start($self->{field}, FALSE, FALSE, 5);

    $self->{rules} = Gtk2::ComboBox->new_text();
    $self->pack_start($self->{rules}, FALSE, FALSE, 5);
    $self->set_rules(\@RR_ALLRULES);

    $self->{text} = Gtk2::Entry->new();
    $self->pack_start($self->{text}, TRUE, TRUE, 5);

    $self->{channelscombobox} = Gtk2::ComboBox->new_text();
    $self->{categoriescombobox} = Gtk2::ComboBox->new_text();

    my $remove = Gtk2::Button->new_from_stock('gtk-remove');
    $remove->signal_connect(clicked => \&on_remove_clicked, $self);
    $self->pack_end($remove, FALSE, FALSE, 5);
    
#	foreach my $c (@::categories) {
#	    $self->{categoriescombobox}->append_text($c);
#	}
#	$self->{categoriescombobox}->set_active(0);
}

sub SET_PROPERTY {
    my ($self, $pspec, $newval) = @_;

    $self->{$pspec->get_name} = $newval;

    if ($pspec->get_name() eq "channels") {
      # Initialize from globals
	foreach my $c (sort { $a->{order} <=> $b->{order} } values %{$self->{channels}}) {
	    next unless ($c->{'visible'});
#	    print "Setting channel " . Dumper($c) . "\n";
	    $self->{channelscombobox}->append_text($c->{name});
	}
	$self->{channelscombobox}->set_active(0);
    }
}

sub on_remove_clicked {
    my ($button, $self) = @_;
    $self->get_parent()->remove($self);
}

sub on_change {
    my ($combo, $self) = @_;

	$self->do_change($combo->get_active_text());
}

# Do when the rule changes
sub do_change {
	my ($self, $field) = @_;

	my @c = caller 1;
	my @c2 = caller 2;
	
#	print "ON_CHANGE: $field by " . $c[3] . " by " . $c2[3] . "\n";	
	$field = get_untranslated_text(\@RR_FIELDS, $field);
#	print "untranslated: $field\n";	
	
    if ($field eq "channel") {
	if ($self->{rulecount} != scalar (@RR_SMALLRULES)) {
	    $self->set_rules(\@RR_SMALLRULES);
	}

	$self->change_entry($self->{channelscombobox});
	
	return;
    }

    if ($field eq "category") {
	if ($self->{rulecount} != scalar (@RR_SMALLRULES)){
	    $self->set_rules(\@RR_SMALLRULES);
	}

	$self->change_entry($self->{categoriescombobox});

	return;
    }

    if ($field eq "title") {
	if ($self->{rulecount} != scalar (@RR_ALLRULES)) {
	    $self->set_rules(\@RR_ALLRULES);
	}

	$self->change_entry($self->{text});

	return;
    }

    if ($field eq "desc") {
	if ($self->{rulecount} != scalar (@RR_ALLRULES)) {
	    $self->set_rules(\@RR_ALLRULES);
	}

	$self->change_entry($self->{text});

	return;
    }
}

sub change_entry {
    my ($self, $entry) = @_;

    my $old = ($self->get_children())[2];

#    print "Change entry: $self from $old to $entry\n";

    if ($old ne $entry) {
#    	print "Changin entry\n";
		$self->remove($old);
		$self->pack_start($entry, TRUE, TRUE, 5);
		$self->reorder_child($entry, 2);
		$self->show_all();
    } else {
#    	print "Not changing\n";
    }
}

sub set_rules {
    my $self  = shift;
    my $rules = shift;

    for (my $i = $self->{rulecount}; defined($i) && $i >= 0 ; $i--) {
	$self->{rules}->remove_text($i);
    }

    foreach my $rule (get_translated_text($rules)) {
	$self->{rules}->append_text($rule);
    }

    $self->{rulecount} = scalar (@$rules);
    $self->{activerules} = $rules;

    $self->{rules}->set_active(0);
}

sub get_data {
    my $self = shift;

    my $p;

    $p->{field} = get_untranslated_text(\@RR_FIELDS, $self->{field}->get_active_text());
    $p->{rule}  = get_untranslated_text($self->{activerules}, $self->{rules}->get_active_text());

    my $value = ($self->get_children())[2];

    if (UNIVERSAL::isa($value, 'Gtk2::Entry')) {
		$p->{value} = $value->get_text();
    } else {
    	if ($p->{field} eq "channel") {
    		$p->{value} = $self->find_channel_by_name($value->get_active_text());
    	} else {
   			$p->{value} = $value->get_active_text();
    	}
    }
    $p;
}

sub set_data {
	my $self = shift;
	my $rule = shift;
		
	for (my $i = 0; $i < scalar (@RR_FIELDS); $i++) {
		if ((keys %{$RR_FIELDS[$i]})[0] eq $rule->{field}) {
			$self->{field}->set_active($i);
			last;
		}
	}

	for (my $i=0; $i < $self->{rulecount}; $i++) {
		if ((keys %{$self->{activerules}[$i]})[0] eq $rule->{rule}) {
			$self->{rules}->set_active($i);
			last;
		}
	}
	
    my $entry = ($self->get_children())[2];

    if (UNIVERSAL::isa($entry, 'Gtk2::Entry')) {
		$entry->set_text($rule->{value});
    } else {
		my $model = $entry->get_model();
		my $i = 0;

		my $val = $rule->{value};

#                print "My val is $val\n";
		
		if ($rule->{field} eq "channel") {
			$val = $self->{channels}->{$val}->{name};
		}	

#		print "And got mapped to $val\n";

		$model->foreach(sub {
			my ($model, $path, $iter, $value) = @_;
			if ($model->get($iter) eq $value) {
				$entry->set_active($i);
				TRUE;
			} else {
				$i++;
				FALSE;
			}
		}, $val);
	}
}

sub get_translated_text {
	my $list= shift;
	
	my @a = ();
	
	foreach my $b (@$list) {
		push @a, (values %$b)[0];
	}
	@a;
}

sub get_untranslated_text {
	my $list = shift;
	my $text = shift;
	
	foreach my $b (@$list) {
		if ((values %$b)[0] eq $text) {
			return (keys %$b)[0];
		}
	}
	undef;
}

sub find_channel_by_name {
        my $self = shift;
	my $name = shift;

	foreach my $c (values %{$self->{channels}}) {
		if ($c->{name} eq $name) {
			return $c->{id};
		}
	}
}
1;

